-- pc speaker tools
-- J. Craig Gilbert
-- cgilbert@cennet.mc.peachnet.edu
-- Change made 11-01-97:
--    Routines no longer rely on an altered clock tick interrupt rate.
-- Instead, they use a for-loop dely tailored at start-up to the current
-- machine.
include graphics.e
include machine.e
include wildcard.e
-- local includes
include string.e

------------------------------------------------------------------------------
-- note equivalencies
-- letter with an underscore after it is a sharp
global constant OCT1 = 1, OCT2 = 2, OCT3 = 3, OCT4 = 4, OCT5 = 5, OCT6 = 6,
                OCT7 = 7, OCT8 = 8,
                C = 1, C_ = 2, D = 3, D_ = 4, E = 5, F = 6,
                F_ = 7, G = 8, G_ = 9, A = 10, A_ = 11, B = 12,
                PAUSE = 0
                
global constant notes ={{   0,   0,   0,  39,  41,  44,  46,  49,  52,  55,  58,  62},
                        {  65,  69,  73,  78,  82,  87,  93,  98, 104, 110, 117, 123},
                        { 131, 139, 147, 156, 165, 175, 185, 196, 208, 220, 233, 247},
                        { 262, 277, 294, 311, 330, 349, 370, 392, 415, 440, 466, 494},
                        { 523, 554, 587, 622, 659, 698, 740, 784, 831, 880, 932, 988},
                        {1047,1109,1175,1245,1329,1397,1480,1568,1661,1760,1865,1975},
                        {2093,2217,2349,2489,2637,2794,2960,3136,3322,3520,3729,3951},
                        {4186,   0,0,0,0,0,0,0,0,0,0,0},
                        {0,0,0,0,0,0,0,0,0,0,0,0}}
------------------------------------------------------------------------------
atom mTime_per_loop
procedure init_timer()
atom t
   t = time()
   for x = 1 to 1000000 do
   end for
   t = time() - t
   mTime_per_loop = t/1000000
end procedure
------------------------------------------------------------------------------
procedure wait_time(atom delay)
-- delay should be in seconds
atom loops
   loops = floor(delay/mTime_per_loop)
   for x = 1 to loops do
   end for
end procedure
------------------------------------------------------------------------------
global procedure tsound(atom frequency, atom delay)
   sound(frequency)
   wait_time(delay)
end procedure
------------------------------------------------------------------------------
global procedure play(sequence freqlist)
-- play a series of frequencies for the specified times
-- freqlist format: {freq, time, freq, time, freq, time, . . .}
-- time is in seconds
   for x = 1 to length(freqlist) by 2 do
      tsound(freqlist[x],freqlist[x+1])
   end for
   sound(0)
end procedure
------------------------------------------------------------------------------
global function freq(integer octave, integer note)
   return notes[octave][note]
end function
------------------------------------------------------------------------------
function get_notelength(integer style, integer tempo, integer notesize)
atom notelen

            if style = 1 then -- legato 
               notelen = (240/tempo)/notesize
            elsif style = 2 then -- normal
               notelen = ((240/tempo)/notesize)*(.9)
            elsif style = 3 then -- staccato
               notelen = ((240/tempo)/notesize)*(.5)
            end if

   return notelen
end function
------------------------------------------------------------------------------
global procedure qplay(sequence s)
-- qplay()
-- plays QBASIC music command strings; supports all QBASIC music notation
-- except MF(foreground play) and MB(background play) - this function 
-- will always play in the foreground
-- Background play on the pc speaker involves machine code and hooking 
-- interrupts, which is out of my range for the moment.
-- The only other difference is that this function will stop on an
-- ESCAPE key press.
atom notelen
integer note, pos, oct, noterep, len, notesize, tempo, temp, noteindex,
        style, tempoct

   style = 2 -- normal
   tempo = 32
   notesize = 4
   notelen = .1
   oct = 2
   len = length(s)
   pos = 1
   while pos <= len do
      noterep = lower(s[pos])
      if noterep = 'o' then
         if pos < len then
            pos = pos + 1
            if s[pos] >= '0' and s[pos] <= '6' then
               oct = s[pos] - '0' + 2  
            end if
         end if
         pos = pos + 1
      elsif noterep = '>' then
         if oct < 8 then
            oct = oct + 1
         end if
         pos = pos + 1
      elsif noterep = '<' then
         if oct > 2 then
            oct = oct - 1
         end if
         pos = pos + 1
      elsif noterep = 't' then
         if pos < len then
            temp = str_to_int(s[pos + 1..len])
            if temp != 0 then
               if temp >= 32 and temp <= 255 then
                  tempo = temp
               end if
            end if
            notelen = get_notelength(style, tempo, notesize)
            pos = pos + 1
            while s[pos] >= '0' and s[pos] <= '9' do
               pos = pos + 1
               if pos > len then
                  exit
               end if
            end while
         else
            pos = pos + 1
         end if
      elsif noterep = 'l' then
         if pos < len then
            temp = str_to_int(s[pos + 1..len])
            if temp != 0 then
               notesize = temp
            end if
            notelen = get_notelength(style, tempo, notesize)
            pos = pos + 1
            while s[pos] >= '0' and s[pos] <= '9' do
               pos = pos + 1
               if pos > len then
                  exit
               end if
            end while
         else
            pos = pos + 1
         end if
      elsif noterep = 'm' then
         if pos < len then
            pos = pos + 1
            if lower(s[pos]) = 'l' then
               style = 1
            elsif lower(s[pos]) = 'n' then
               style = 2
            elsif lower(s[pos]) = 's' then
               style = 3
            else -- F or B, this function doesn't support background play
               -- do nothing
            end if
            notelen = get_notelength(style, tempo, notesize)
         end if
         pos = pos + 1
      elsif noterep = 'p' then
         if pos < len then
            temp = str_to_int(s[pos + 1..len])
            if temp != 0 then
               notesize = temp
               tsound(PAUSE,(240/tempo)/notesize)
               pos = pos + 1
               while s[pos] >= '0' and s[pos] <= '9' do
                  pos = pos + 1
                  if pos > len then
                     exit
                  end if
               end while
            end if       
         else
            pos = pos + 1
         end if
      else
         if noterep = 'c' then
            noteindex = C
         elsif noterep = 'd' then
            noteindex = D
         elsif noterep = 'e' then
            noteindex = E
         elsif noterep = 'f' then
            noteindex = F
         elsif noterep = 'g' then
            noteindex = G
         elsif noterep = 'a' then
            noteindex = A
         elsif noterep = 'b' then
            noteindex = B
         elsif noterep = 'n' then
            if pos < len then
               temp = str_to_int(s[pos + 1..len])
               if temp != 0 then
                  if remainder(temp,12) != 0 then
                     noteindex = remainder(temp,12)
                     tempoct = floor(temp/12) + 2
                  else
                     noteindex = (temp/12)
                     tempoct = floor(temp/12) + 1
                  end if
                  notesize = temp
                  pos = pos + 1
                  while s[pos] >= '0' and s[pos] <= '9' do
                     pos = pos + 1
                     if pos > len then
                        exit
                     end if
                  end while
               else  -- 0 is pause for N command form
                  noteindex = 0
               end if       
            end if
         else
            noteindex = 0
         end if

         if not noteindex then
            note = PAUSE
         elsif noterep = 'n' then
            if temp != 0 then 
               note = freq( tempoct , noteindex )
            end if
         elsif pos < len then
            if s[pos+1] = '-' then
               if noterep = 'c' then
                  note = freq(oct-1,B)
               else
                  note = freq(oct,noteindex - 1)
               end if
               pos = pos + 1
            elsif find(s[pos+1],"#+") then
               if noterep = 'b' then
                  note = freq(oct+1,C)
               else
                  note = freq(oct,noteindex + 1)
               end if
               pos = pos + 1
            else
               note = freq(oct,noteindex)
            end if
         else
            note = freq(oct,noteindex)
         end if

         if pos < len then
            if s[pos + 1] = '.' then
               tsound(note,(notelen*1.5))
               pos = pos + 1
            else
               tsound(note,notelen)
            end if
         else
            tsound(note,notelen)
         end if
         
         if style = 1 then -- legato
            -- no pause
         else -- normal OR staccato
            tsound(PAUSE, notelen/9 )
         end if
         pos = pos + 1
      end if
      if get_key() = 27 then -- escape key pressed
         exit
      end if
   end while

   sound(0)
end procedure
------------------------------------------------------------------------------
------------------------------------------------------------------------------
------------------------------------------------------------------------------
------------------------------------------------------------------------------
init_timer()

