-- J. Craig Gilbert
-- cgilbert@cennet.mc.peachnet.edu
include wildcard.e
include get.e
------------------------------------------------------------------------------
------------------------------------------------------------------------------
------------------------------------------------------------------------------
global function expand_tabs(sequence s,atom tspaces)
integer startpos, tabloc, endpos
sequence tab

   tab = repeat(' ',tspaces)
      
   if length(s) = 0 then
      return s
   elsif length(s) = 1 then
      if s[1] = '\t' then
         s = tab
         return s
      end if
   elsif s[length(s)] = '\t' then
      s = s[1..length(s)-1] & tab
   elsif s[1] = '\t' then
      s = tab & s[2..length(s)]
   end if
   
   startpos = 2
   endpos = length(s)
   tabloc = find('\t',s)
   while tabloc do
      s = s[1..tabloc-1] & tab & s[tabloc+1..endpos]
      startpos = tabloc + tspaces
      endpos = endpos + tspaces - 1
      tabloc = 0
      for x = startpos to endpos do
         if s[x] = '\t' then
            tabloc = x
            exit
         end if
      end for
   end while

   return s
end function
------------------------------------------------------------------------------
global function get_to_char(sequence s,atom start, atom stopchar)
-- return a string starting with sequence s at the given position and
-- continuing up to the first occurance of stopchar; the stopchar is
-- left out.
integer slen, charpos
sequence out

   out = {}
   slen = length(s)

   charpos = find(stopchar,s[start..slen])
   if charpos then
      if charpos > start then
         out = s[start..charpos-1]
      end if
   else
      out = s[start..slen]
   end if

   return out
end function
------------------------------------------------------------------------------
global function trim(sequence s, atom trimchar)
-- trim all instances of given character from both ends of sequence s
integer len, pos

   len = length(s)
   if len then
      pos = 1
      while s[pos] = trimchar do
         if pos = len then
            exit
         end if
         pos = pos + 1
      end while

      s = s[pos..len]
      len = length(s)
      pos = len
      while s[pos] = trimchar do
         if pos = 1 then
            exit
         end if
         pos = pos - 1
      end while
      s = s[1..pos]
      if length(s) = 1 then
         if s[1] = trimchar then
            s = ""
         end if
      end if
   end if

   return s
end function
------------------------------------------------------------------------------
global function left(sequence s, atom len)
   if len < 1  then
      s = ""
   elsif length(s) > len then
      s = s[1..len]
   end if

   return s
end function
------------------------------------------------------------------------------
global function right(sequence s, atom len)
integer origlen
   origlen = length(s)
   if len < 1  then
      s = ""
   elsif origlen > len then
      s = s[origlen-len+1..origlen]
   end if

   return s
end function
------------------------------------------------------------------------------
global function mid(sequence s, atom len, atom start)
integer origlen
   origlen = length(s)
   if len < 1 then
      s = ""
   elsif start <= origlen then
      if (start + len - 1) > origlen then
         s = s[start..origlen]
      else
         s = s[start..start+len]
      end if
   else
      s = ""
   end if
   return s
end function
------------------------------------------------------------------------------
global function fitstring(sequence s, atom len, integer padchar)
-- make string fit length by clipping or padding
-- padding is done with the character specified
integer origlen
   origlen = length(s)
   if origlen = len then
      -- do nothing
   elsif origlen > len then
      s = left(s,len)
   else -- less than len
      s = s & repeat(padchar,len-origlen)
   end if
   return s
end function
------------------------------------------------------------------------------
global function hexchar_to_atom(atom c)
-- returns -1 for error
   c = lower(c)
   return (find(c,"0123456789abcdef")-1)
end function
------------------------------------------------------------------------------
global function hexstr_to_atom(sequence h)
-- returns a sequence for error, else a number
atom n, hlen, signfix, start, placeval
   
   n= 0
   h = trim(h,' ')
   hlen = length(h)
   if hlen = 0 then
      return {}
   end if
   if h[1] = '-' then
      signfix = -1
      start = 2
   else
      signfix = 1
      start = 1
   end if
   placeval = 1
   for x = hlen to start by -1 do
      if not find(h[x],"0123456789ABCDEFabcdef") then
         return {}
      end if
      n = n + hexchar_to_atom(h[x])*placeval
      placeval = placeval*16
   end for

   return (n*signfix)   
end function
------------------------------------------------------------------------------
global function match_first_letter(sequence list, integer start, integer letter)
-- return the index of the first string in list whose first letter matches
-- 'letter', starting immediately after the index given by 'start'
-- A start index less than 1 will be taken to mean start at one.
integer found
   if start < 1 then
      start = 0
   end if
   found = 0
   for x = start+1 to length(list) do
      if length(list[x]) then
         if list[x][1] = letter then
            found = x
            exit
          end if
      end if
   end for
   return found
end function
------------------------------------------------------------------------------
global function str_to_int(sequence s)
-- translate a string of numerical characters into an integer
-- ignore leading space
-- stops translating when it hits a non-digit and returns whatever it got
-- by that point
object num
integer len
sequence numstr

   s = trim(s,' ')
   len = length(s)
   numstr = "" 
   for x = 1 to len do
      if s[x] >= '0' and s[x] <= '9' then
         numstr = numstr & s[x]
      else
         exit
      end if
   end for

   num = value(numstr)
   if num[1] = GET_SUCCESS then
      return num[2]
   else
      return 0
   end if
end function
------------------------------------------------------------------------------

